/**
* @class StreetView - Javacriptová verze (v3)
* @group StreetView
*/

StreetView = JAK.ClassMaker.makeClass({
NAME:"StreetView",
VERSION:"1.0"
});

/**
* Konstruktor
* @param {SMap} map Mapa
* @param {String} width Šířka okna (25%, 50%, 75%, 100%)
* @param {SMap.Coords} coords Souřadnice startu StreetView
* @param {Int} heading Heading   
*/

StreetView.prototype.$constructor = function(map, width, coords, heading)
{
  this.coords = coords;
  this.heading = heading || 0;

  this.svWidth = width || 50;  /* width of StreetView div in percent */
  this.svWidthStep = 25;       /* resize button step */
  this.map = map;
  this.mapDom = this.map.getContainer();

  this._sc = [];   /* handlers for map events */
  this._ec = [];   /* JAK.Events handlers */

  // create container
  this.streetViewDiv = JAK.mel("div", {className: "StreetViewDiv"});
  this.mapDom.parentNode.insertBefore(this.streetViewDiv, this.mapDom.nextSibling);
  this._setHeight();

  // create map arrow
  this.arrowLayer = new SMap.Layer.Marker();
  this.map.addLayer(this.arrowLayer);
  this.arrowLayer.enable();  
  
  this.arrowDiv = JAK.mel("div", {className: "StreetViewMarker"});
  this.arrowMarker = new SMap.Marker(coords, null, {anchor: {left: 17,top: 19}, url: this.arrowDiv});
  this.arrowMarker.decorate(SMap.Marker.Feature.Draggable);
  this.arrowLayer.addMarker(this.arrowMarker);
  this.updateArrowPosition();
  
  // handler for dragging position arrow (which result in changing our position) 
  this._sc.push(this.map.getSignals().addListener(this, "marker-drag-stop", "_onArrowDragStop"));
  
  // handler for clicking on arrow (changes heading)
  this._ec.push(JAK.Events.addListener(this.arrowDiv, "click", this, "_onArrowClick", true));  
  
  // handler for keyboard input
  this._ec.push(JAK.Events.addListener(this.streetViewDiv, "keydown", this, "_onKeyDown", true));

  // handler for window resize, adjust StreetView size
  this._ec.push(JAK.Events.addListener(window, "resize", this, "_onWindowResize", true));
  
  // handler for map redraw (to set correct arrow angle when orientation is not 0°)
  this._sc.push(this.map.getSignals().addListener(this, "map-redraw", "_onMapRedraw"));

  // load StreetView at given coords
  //this._loadStreetView();
  window.setTimeout(function(){ this._loadStreetView(); }.bind(this), 0); /* this prevents problems with starting without dom updates being finished */

  // show StreetView window
  this.showStreetView();
}

/**
* Destruktor
*/

StreetView.prototype.$destructor = function()
{
  // unload StreetView first
  this._unloadStreetView();

  // remove handlers
  this.map.getSignals().removeListeners(this._sc);
  JAK.Events.removeListeners(this._ec);

  // remove arrow layer
  this.arrowLayer.disable();
  this.arrowLayer.removeAll();
  this.map.removeLayer(this.arrowLayer);
  this.arrowLayer.$destructor();
  
  // hide StreetView window
  this.hideStreetView();
  
  // remove main div and restore map style
  this.streetViewDiv.parentNode.removeChild(this.streetViewDiv);
  this.mapDom.style.width = "";
  this.mapDom.style.display = "";  
  
  // broadcast exit
  this.map.getSignals().makeEvent("street-view-resize", this, {newWidth: this.svWidth}); 
  
  // sync map
  this._syncMap();
}

/**
* odstraní z divu Street View
*/

StreetView.prototype._unloadStreetView = function()
{        
  // remove resize, close and fullscreen buttons
  [this.svFullscreenButton, this.svCloseButton, this.svResizeButton].forEach(function(button)
  {
    if (typeof button !== 'undefined')
    {
      button.parentNode.removeChild(button);
    }
  });
}

/**
* Načte do připraveného divu Street View
*/

StreetView.prototype._loadStreetView = function()
{
  this.arrowMarker.setCoords(this.coords);
  this._createResizeButton();
}

/**
* Zjistí, zda je StreetView aktivní (tedy je div otevřený)
  @returns {boolean} Zda je okno se StreetView otevřené
*/

StreetView.prototype.isActive = function()
{
  return contains_class(document.documentElement, "streetview_on");  
}

/**
* Zobrazí skrytý div se StreetView
*/

StreetView.prototype.showStreetView = function()
{
  add_class(document.documentElement, "streetview_on");  
}

/**
* Skryje div se StreetView
*/

StreetView.prototype.hideStreetView = function()
{
  remove_class(document.documentElement, "streetview_on");  
}

/**
* synchronizace mapy
*/

StreetView.prototype._syncMap = function()
{
  // setTimeout je hack, aby StreetView Embed v GeoSetteru se nevracelo někam na předchozí pozici
  // -- příčina bugu neznámá, někde v SMap.prototype.syncPort
  window.setTimeout(function()
  {
    this.map.syncPort();
    window.createDefaultMapControls(); /* nesystémové :( */

  }.bind(this),0);
}

/**
* Nastaví výšku okna (nejde přes CSS)
*/

StreetView.prototype._setHeight = function()
{
  // IE8
  if (!this.mapDom.offsetHeight)
    this.streetViewDiv.style.height = (this.mapDom.clientHeight + 2 /* IE vrací o 2px méně než u offsetWidth */) + "px";
  // jinde
  else
    this.streetViewDiv.style.height = this.mapDom.offsetHeight + "px";
}

/**
* Přesune se na nové souřadnice (podtřída mění pozici StreetView)
* @param {SMap.Coords} coords Souřadnice
* @param {Int} heading Heading
*/

StreetView.prototype.updatePosition = function(coords, heading)
{
  if (typeof coords !== 'undefined')
    this.coords = coords;
  
  if (typeof heading !== 'undefined')
    this.heading = heading;
    
  this.updateArrowPosition();
}

/**
* Nastavíme orientaci šipky
*/

StreetView.prototype.updateArrowAngle = function()
{
  // nastavíme úhel
  // -- je třeba brát v úvahu i otočení mapy
  var angle = this.heading + 90 * this.map.getOrientation() /* 0 = 0°, 1 = 90°, 2 = 180°, 3 = 270° */;
  rotateElement(this.arrowDiv, angle);
}

/**
* Nastavíme polohu a orientaci šipky dle aktuální polohy
*/

StreetView.prototype.updateArrowPosition = function()
{
  this.arrowMarker.setCoords(this.coords);
  this.updateArrowAngle();
  
  // nastavíme mapu aby šipka byla uprostřed
  this.map.setCenter(this.coords);
}

/**
* Změní velikost divu obsahujícího StreetView, a to pomocí tlačítka (např. 25%, 50%, 75%, 100%)
*/

StreetView.prototype._resizeWidth = function()
{
  // set new width
  this.svWidth = (this.svWidth + this.svWidthStep) % 100;
  this.svWidth = this.svWidth || 100; /* because 100 mod 100 is 0 */

  // update width and button value
  this._setWidth();

  // broadcast this (so it can be saved to settings by main program)
  this.map.getSignals().makeEvent("street-view-resize", this, {newWidth: this.svWidth});
}

/**
* Nastaví šířku obou divů (mapy a StreetView) a také hodnotu resizovacího tlačítka
*/

StreetView.prototype._setWidth = function()
{
  this.mapDom.style.width = (100 - parseInt(this.svWidth)) + "%";     /* set map width to 100 minus StreetView width */
  this.mapDom.style.display = (this.svWidth == 100) ? "none" : "";    /* turn off the map when it takes 0% */

  this.streetViewDiv.style.width = this.svWidth + "%";                /* set StreetView div (always after shrinking the map above) */
                                                                      
  this.svResizeButton.innerHTML = this.svWidth + "%";
  
  // synchronize map
  this._syncMap();
}

/**
* Synchronizace okna se StreetView (provedeme kompletní restart)
* -- je voláno vnějšími funkcemi, pokud nějak překopou rozměry prvků 
*/

StreetView.prototype.syncViewPort = function()
{
  // ukončíme, pokud Street View není aktivní
  if (!this.isActive())
    return;
  
  // fixes some problems with div switcheroo in on/off allscreen functions (completely restructure all the divs later)
  this.mapDom.parentNode.insertBefore(this.streetViewDiv, this.mapDom.nextSibling);
  
  this._onWindowResize();

//   // unloadneme komplet vše
//   if (this.isActive())
//     this.$destructor();
// 
//   // znovu vše načteme  
//   this.$constructor(this.map, this.svWidth, this.coords, this.heading);
//   
//   // -- následující nevolat pro Embedded, protože tam se volá unload a to dřív, než se stihne zavolat load (kvůli timeoutu!)
//   this.updatePosition(this.coords, this.heading);
}

/**
* Vytvoření tlačítka pro změnu velikost divu se StreetView
*/

StreetView.prototype._createResizeButton = function()
{
  this.svResizeButton = JAK.mel("div", {className: "button sidebar_button sv_size_button"});  
  this.streetViewDiv.appendChild(this.svResizeButton);
  
  this._ec.push(JAK.Events.addListener(this.svResizeButton, "click", this, "_onClickingResizeButton")); 

  this._setWidth();
}

/**
* Vytvoření tlačítka pro přepnutí do fullscreenu
*/

StreetView.prototype._createFullscreenButton = function()
{
  // pokud není full screen mód podporován, tak nemá smysl tlačítko vytvářet
  if (!screenfull.enabled)
    return;

  // fullscreen doesn't work in embedded mode
  if (JAK.Browser.embedded)
    return;

  // create button
  this.svFullscreenButton = JAK.mel("div", {className: "sv_fullscreen_button"});
  this.streetViewDiv.appendChild(this.svFullscreenButton);

  // enter or exit fullscreen upon clicking the button
  this._ec.push(JAK.Events.addListener(this.svFullscreenButton, "click", this, "_onClickingFullscreenButton")); 
}

/**
* Vytvoření tlačítka pro uzavření StreetView
*/

StreetView.prototype._createCloseButton = function()
{
  // create button
  this.svCloseButton = JAK.mel("div", {className: "sv_close_button"});  
  this.streetViewDiv.appendChild(this.svCloseButton);
  
  // exit on clicking the button
  this._ec.push(JAK.Events.addListener(this.svCloseButton, "click", this, "_onClickingCloseButton")); 
}

/**
* Handler pro resizování okna
*/

StreetView.prototype._onWindowResize = function(e, elm)
{
  this._setHeight();
  this._setWidth();
  this._syncMap();
}

/**
* Handler pro překreslení mapy (kvůli nastavení orientace šipky, pokud není orientace mapy 0°)
*/

StreetView.prototype._onMapRedraw = function(e, elm)
{
  this.updateArrowAngle();
}

/**
* Handler pro vstup z klávesnice
*/

StreetView.prototype._onKeyDown = function(e, elm)
{
//   // ENTER is ok
//   if (e.keyCode == 13)
//     return;
// 
//   EventUtil.preventDefault(e);
//   EventUtil.stopPropagation(e);
}


/**
* Handler pro tažení šipky ukazující polohu
*/

StreetView.prototype._onArrowDragStop = function(e, elm)
{
  if (e.target !== this.arrowMarker)
    return;
  
  this.updatePosition(e.target.getCoords(), this.heading);
}

/**
* Handler pro kliknutí na šipku (změníme azimut)
*/

StreetView.prototype._onArrowClick = function(e, elm)
{
  // zjistíme, zda se opravdu jednalo jen o klik nebo zda jsme místo něj šipkou táhli
  if (this.arrowMarker._wasDrag)
    return;

  // zjistíme souřadnice kliku a středu šipky  
  var clickCoords = SMap.Coords.fromEvent(e, this.map);
  var arrowCenterCoords = this.coords;
  
  // spočteme azimut od středu k bodu kliku - to je náš hledaný heading
  this.heading = arrowCenterCoords.azimuth(clickCoords);
  
  // nastavíme pozici
  this.updatePosition(this.coords, this.heading);
}

/**
* Handler pro kliknutí na tlačítko pro změnu velikosti divu se StreetView
*/

StreetView.prototype._onClickingResizeButton = function()
{
  this._resizeWidth();
}

/**
* Handler pro kliknutí na tlačítko pro fullscreen
*/

StreetView.prototype._onClickingFullscreenButton = function()
{
  if (screenfull.enabled)
    screenfull.toggle(this.streetViewDiv);
}

/**
* Handler pro kliknutí na tlačítko pro zavření
*/

StreetView.prototype._onClickingCloseButton = function()
{
  // exit fullscreen first, if it is active
  if (screenfull.enabled)
    screenfull.exit();  

  this.$destructor();
}

//***************************************************************************************
//                                Street View Embed
//***************************************************************************************

/**
* @class StreetView - Embedded verze
* @group StreetView
*/

StreetView.Embedded = JAK.ClassMaker.makeClass({
NAME:"StreetView.Embedded",
VERSION:"1.0",
EXTEND: StreetView,
IMPLEMENT: [SMap.IOwned, JAK.IDecorable]
});

/**
* Konstruktor
* @param {SMap} map Mapa
* @param {String} width Šířka okna (25%, 50%, 75%, 100%)
* @param {SMap.Coords} coords Souřadnice startu StreetView
* @param {Int} heading Heading   
*/
StreetView.Embedded.prototype.$constructor = function(map, width, coords, heading)
{
  this.$super.apply(this, arguments);
}

/**
* odstraní z divu Street View
*/

StreetView.Embedded.prototype._unloadStreetView = function()
{
  this.$super();

  // remove iframe
  this.streetViewDiv.removeChild(this.streetViewDiv.firstChild);
  
  remove_class(document.documentElement, "svEmbedded");  
}

/**
* Načte do připraveného divu Street View
*/

StreetView.Embedded.prototype._loadStreetView = function()
{
//   // http://asnsblues.blogspot.cz/2011/11/google-maps-query-string-parameters.html 
//   // -- verze, která nepotřebuje key, ale nepodporuje heading 
//   this.streetViewDiv.innerHTML = "<iframe class=sviFrame "
//    + "frameborder=0 scrolling=no marginheight=0 marginwidth=0 "   /* these can't be set through CSS */
//    + "src=\"http://maps.google.com/maps/sv?" 
//    + "&cbll=" + this.coords.lat() + "," + this.coords.lon() + "&hl=cs\"></iframe>";

  // https://developers.google.com/maps/documentation/embed/guide#street_view_mode
  this.streetViewDiv.innerHTML = "<iframe class=sviFrame "
   + "frameborder=0 scrolling=no marginheight=0 marginwidth=0 "    /* these can't be set through CSS */
   + "src=\"https://www.google.com/maps/embed/v1/streetview?key=AIzaSyCbI2JzzxEwHomn7Og_xxk5oUIlPTCmKEg"
   + "&location=" + this.coords.lat() + "," + this.coords.lon() + "&heading=" + this.heading + "\"></iframe>";

  this._createCloseButton();
  this._createFullscreenButton();
  
  add_class(document.documentElement, "svEmbedded"); 
  
  this.$super();
}

/**
* Přesune se na nové souřadnice
* @param {SMap.Coords} coords Souřadnice
* @param {Int} heading Heading
*/

StreetView.Embedded.prototype.updatePosition = function(coords, heading)
{
  this.$super(coords, heading);

  // need full reload here :(
  this._unloadStreetView();
  this._loadStreetView();
}

//***************************************************************************************
//                                Street View Flash (v2)
//***************************************************************************************

// podpora:
// - je třeba (pro protokol file:) přidat adresář s mapou v nastavení Flashe zde:
//   http://www.macromedia.com/support/documentation/en/flashplayer/help/settings_manager04.html
// - protokole file bez nastavení výše: StreetView se zasekne při pokusu se někam pohnout
// - protokole file s nastavením výše: vše funguje jak má (kromě Firefoxu)
// - protokol http: funguje bez nastavování, ale nefunguje možnost volat z javascriptu flashové funkce (tedy i nastavit sépiový mód)

/**
* @class StreetView.Flash - Flashová verze (v2)
* @group StreetView.Flash
*/

StreetView.Flash = JAK.ClassMaker.makeClass({
NAME:"StreetView.Flash",
VERSION:"1.0",
EXTEND: StreetView,
IMPLEMENT: [SMap.IOwned, JAK.IDecorable]
});

/**
* odstraní z divu Street View
*/

StreetView.Flash.prototype._unloadStreetView = function()
{
  this.$super();
 
  // odstraníme flashový objekt
  this.flashInterface.parentNode.removeChild(this.flashInterface);

  // odstraníme flag pro CSS
  remove_class(document.documentElement, "svFlash");
  
  // odstraníme tlačítko pro sépiový mód
  this.svSepiaModeButton.parentNode.removeChild(this.svSepiaModeButton);

  // odstraníme callback pro flash
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
  {
    window["GStreetviewFlashCallback_" + this.flashCallbackId] = null;
    return;
  }
 
  delete window["GStreetviewFlashCallback_" + this.flashCallbackId];
}

/**
* Načte do připraveného divu Street View
*/

StreetView.Flash.prototype._loadStreetView = function()
{
  // load google.maps only once and never unload (Google maps don't support unloading)
  if (!window.google)
  {
    // nastavíme globální proměnnou pro callback a hned ji po spuštění opět odstraníme
    window.gMapsLoad = this;
    loadJS("http://maps.googleapis.com/maps/api/js?sensor=false&callback=gMapsLoad._loadStreetView");
    return;
  }
          
  // zjistíme panorama ID (potřebujeme ho zadat do parametrů pro vytvoření Flashe) a opět z funkce utečeme než to bude
  else if (!this.gStreetViewService)
  {
    // zrušíme pomocnou globální proměnnou
    window.gMapsLoad = null;  

    // https://developers.google.com/maps/documentation/javascript/streetview
    this.gStreetViewService = new google.maps.StreetViewService();
    
    
    this.getPanoIdFromCoords(this.coords, function(found, panoId, coords)
    {
      // pokud jsme našli panorama, tak nastavíme jeho souřadnice a panoId
      if (found)
      {
        this.coords = coords;
        this.panoId = panoId;
      }
      // pokud ne, tak souřadnice necháme a panoId nastavíme na prázdný řetězec (pak se zobrazí jen černé pozadí)
      else
        this.panoId = "";
        
      this._loadStreetView();
    }.bind(this));    

    return;  
  }
  
  // vytvoříme callback pro flash (s unikátním ID, aby nebyl problém, kdyby bylo více instancí této třídy)
  this.flashCallbackId = new Date().getTime();
  window["GStreetviewFlashCallback_" + this.flashCallbackId] = function() { this.gFlashCallback.apply(this, arguments); }.bind(this);

  // parametry pro Flash
	var params =
  {
		src: "https://maps.gstatic.com/intl/cs_ALL/mapfiles/cb/googlepano.165.swf",
    //src: basePath + "Images/StreetView/v2/googlepano.166.swf", /* (v lokální verzi nefunguje kolečko uvnitř SV pro posun na další pozici) */
    id: "panoflash" + this.flashCallbackId,
		type: "application/x-shockwave-flash",
	};

  // parametr pro Flash s názvem flashVars
	var flashVars =
      'panoId=' + this.panoId
    + '&yaw=' + this.heading
    + '&pitch=' + 0
    + '&zoom=' + 0

    + '&useSsl=' + 'true'            
    //+ '&directionMap=' + 'N:S,W:Z,S:J,E:V,NW:SZ,NE:SV,SW:JZ,SE:JV'      
    + '&browser=' + 'firefox'
    + '&viewerId=' + this.flashCallbackId
    + '&context=' + 'maps_sv'
    + '&csiCallback=' + '_0i74mx9ip'
    + '&userPhotoRepositories=' + 'all'
    + '&deeplink=' + 0

    // hl: USER_LANGUAGE
    // gl: USER_COUNTRY
    // ku: USE_KENNEDY_UI
    // rhsc: USE_RIGHT_ALIGNMENT
    // ph: USE_LOOKAROUND
    // sv: USE_STREET_VIEW
    // po: PANORAMIO_PHOTO_MODE
    // fs: USE_FULL_SCREEN
    // vd: USE_VIDEO
    // an: USE_ANNOTATIONS
    // ss: USE_SESAME_STREET
    // sns: USE_SNAPSHOTTER
    // ul: USE_UNSNAPPED_LAT_LNG
    // sm: USE_POIS
    // cp: ENABLE_CUBE_PANO
    // np: ENABLE_NEARBY_POI
    // eb: ENABLE_EASTER_EGG_MODE
    // cu: CITYBLOCK_FROG_LOG_USAGE

  + '&rtfArgs=' + 'hl:cs,gl:,an:1,ro:1,ss:1,sm:1,ku:1,fs:1,sv:1,ph:1,np:0,ul:0,eb:1,sns:1';

  // vytvoříme html element s flashem
  var html = '<object name="' + params.id + '" id="' + params.id +
    '" type = "' + params.type + '" width="100%" height="100%" data="' + params.src + '">' +
  	'<param name="allowScriptAccess" value="always" />' +
  	'<param name="scale" value="noScale" />' +
  	'<param name="salign" value="lt" />' +
    '<param name="allowfullscreen" value="true" />' +
    '<param name="swliveconnect" value="false" />' +
    '<param name="wmode" value="opaque" />' +
    '<param name="quality" value="high" />' +            
  	'<param name="bgcolor" value="#000000" />' +            
  	'<param name="movie" value="' + params.src + '" />' +
  	'<param name="flashVars" value="' + flashVars + '" />' +
  '</object>';

//   // možná bude třeba někdy pro staré prohlížeče...
//   html = '<embed name="'+params.id+'" id="'+params.id+'" width="100%" height="100%"' +
// 				'allowScriptAccess="always" ' +
// 				'scale="noScale" ' +
// 				'salign="lt" ' +
//         'allowfullscreen="true" ' +
//         'swliveconnect="false" ' +
//         'wmode="opaque" ' +
//         'quality="high" ' +            
// 				'bgcolor="#000000" ' +            
//         'type="' + params.type + '" ' +
// 				'src="' + params.src + '" ' +
//         'type="' + params.type + '" ' +
// 				'flashVars="' + flashVars + '" ' +
// 			'/>';

  this.streetViewDiv.innerHTML = html;
   
  // rozhraní, které obsahuje funkce exportované Flashem
  this.flashInterface = document.getElementById(params.id);

  // vytvoříme tlačítka
  this._createCloseButton();
  this._createSepiaModeButton();  
  
  add_class(document.documentElement, "svFlash"); 
  
  this.$super();
}

/**
* funkce, kterou volá Flash a předává události
* @param {SMap.Coords} coords Souřadnice
* @param {Int} heading Heading
*/

StreetView.Flash.prototype.gFlashCallback = function(action, param, c, d, e, f, g, h)
{
  if (action == "NewPano" || action == "YawChange" || action == "ZoomChange" || action == "PitchChange")
  {
    // formát args: 50.431675|15.348042|204.68390139312976|502||glyUKYMZ7CsMDK_C6VuwPA|false|0||
    var args = param.args.split("|");
    
    this.coords = SMap.Coords.fromWGS(args[0], args[1]);
    this.heading = param.yaw;
    this.panoId = args[5];

    this.updateArrowPosition();
  }
}

/**
* Zajistíme, aby se šipka nepřesunula dočasně na staré místo, zatímco čekáme, zda je panorama na nových souřadnicích
*/

StreetView.Flash.prototype.updateArrowPosition = function()
{
  if (this.dontUpdateArrowPosition)
    return;
  
  // pokud zrovna nečekáme na souřadnice nového panorama, tak neupdatujeme šipku   
  this.$super();
}

/**
* Přesune se na nové souřadnice
* @param {SMap.Coords} coords Souřadnice
* @param {Int} heading Heading
*/

StreetView.Flash.prototype.updatePosition = function(coords, heading)
{
  this.dontUpdateArrowPosition = true;

  this.$super();
  
  // zavoláme funkci, která zjišťuje nové panoId a po nalezení či nenalezení zavoláme Flash
  this.getPanoIdFromCoords(coords, function(found, panoId, coords)
  {
    // pokud jsme našli nové panoId, tak tam přenastavíme panoId a souřadnice
    if (found)
    {
      this.coords = coords;
      this.panoId = panoId;
    }

    // pokud jsou splněny podmínky níže (kdy nelze interface volat), tak provedeme tvrdý restart
    if (!this.flashInterface.RenderNewPano       /* interface je z nějakého důvodu nedostupný */
    || window.location.protocol != "file:")      /* v http protokolu nelze volat interface, jen přijímat */
    {
      this._unloadStreetView();
      this._loadStreetView();    
    }    

    // zavoláme Flash, aby se přesunul na nové panoId resp. změnil heading
    else if (this.flashInterface.RenderNewPano)
    {
      //this.flashInterface.ChangePOV(this.heading, 0 /*pitch*/, 0 /*zoom*/, false /*should animate*/, false /*use longest route*/);
      this.flashInterface.RenderNewPano(this.panoId, this.heading, 0 /*pitch*/, 0 /*zoom*/);
    }
    
    this.dontUpdateArrowPosition = false;
  }.bind(this));

  // všechny funkce, které je možno volat:
  // RenderNewPano, ChangePOV, FollowLink, HidePano, ChangeZoom, UpdatePanoConfig, SetSepiaMode, SetEasterEggMode (to samé jako SetSepiaMode)
  // AddPhoto, EvalPhotoCommand, SetStreetViewMarkers, OpenStreetViewInfoWindow, CloseStreetViewInfoWindow, UpdateMarker    
}

/**
* Vyžádáme si nové panoId (protože Flashové StreetView nebere souřadnice, jen panoId)
* @param {SMap.Coords} coords Souřadnice
* @param {function} handler Funkce, která bude reagovat na dodané nové panoId a souřadnice
*/

StreetView.Flash.prototype.getPanoIdFromCoords = function(coords, handler)
{
  this.gStreetViewService.getPanoramaByLocation(coords.toGoogle(), 50 /* rádius hledání, musí být <= 50 m */, function(data, status)
  {
    var found = false;
    
    // pokud jsme našli, tak si vytáhneme panoId a nové souřadnice
    // -- kontrolujeme data.links.length, pokud je velikost 0, tak máme uživatelskou fotku, kterou StreetView nezobrazí
    // -- pokud je hodnota 2, tak jsme správně
    if (status == google.maps.StreetViewStatus.OK && data.links.length > 0)
    {
      var panoId = data.location.pano;
      var coords = SMap.Coords.fromGoogle(data.location.latLng);
      found = true;
    }

    // voláme dodaný handler
    handler(found, panoId, coords);
  });
}

/**
* Vytvoření tlačítka pro zapnutí resp. vypnutí Sepia módu
*/

StreetView.Flash.prototype._createSepiaModeButton = function()
{
  this.svSepiaModeButton = JAK.mel("div", {className: "button sidebar_button sv_sepia_button"});  
  this.streetViewDiv.appendChild(this.svSepiaModeButton);
  this.svSepiaModeButton.innerHTML = "Sepia";
  this.sepiaMode = false;

  // v protokolu http nefunguje, takže tlačítko skryjeme  
  this.svSepiaModeButton.style.display = (window.location.protocol == "file:") ? "" : "none";  
  
  // po kliku na tlačítko přehodíme normální mód na sépiový nebo naopak
  // -- a také podle toho přehodíme text tlačítka    
  this._ec.push(JAK.Events.addListener(this.svSepiaModeButton, "click", this, function()
  {
    this.sepiaMode = !this.sepiaMode;
    
    if (this.flashInterface.SetSepiaMode)  /* zkontrolujeme, zda je interface dostupný */
      this.flashInterface.SetSepiaMode(this.sepiaMode /* true - zapnout, false - vypnout */);
    
    this.svSepiaModeButton.innerHTML = !this.sepiaMode ? "Sepia" : "Normal";
  })); 
}

//***************************************************************************************
//                                Street View v3
//***************************************************************************************

// dokumentace pro Street View v3
// https://developers.google.com/maps/documentation/javascript/streetview
// https://developers.google.com/maps/documentation/javascript/reference

/**
* @class StreetView - Javacriptová verze (v3)
* @group StreetView
*/

StreetView.v3 = JAK.ClassMaker.makeClass({
NAME:"StreetView.v3",
VERSION:"1.0",
EXTEND: StreetView,
});

/**
* Konstruktor
* @param {SMap} map Mapa
* @param {String} width Šířka okna (25%, 50%, 75%, 100%)
* @param {SMap.Coords} coords Souřadnice startu StreetView
* @param {Int} heading Heading   
*/

StreetView.v3.prototype.$constructor = function(map, width, coords, heading)
{
  this.$super.apply(this, arguments);
  
  this.gMarkers = []; /* pole markerů (kešek), které umístíme do StreetView */
}

/**
* odstraní z divu Street View
*/

StreetView.v3.prototype._unloadStreetView = function()
{
   this.$super();
   
  // smazání všech markerů
  this.deleteAllMarkers();

  // zrušení všech handlerů
  google.maps.event.clearInstanceListeners(this.gPanorama);

  remove_class(document.documentElement, "svVersion3");
  
  this.streetViewDiv.removeChild(this.streetViewDiv.lastChild);
}

/**
* Načte do připraveného divu Street View
* @param {SMap.Coords} coords Souřadnice startu StreetView
*/

StreetView.v3.prototype._loadStreetView = function()
{  
  // load google.maps only once and never unload (Google maps don't support unloading)
  if (!window.google)
  {
    // nastavíme globální proměnnou pro callback a hned ji po spuštění opět odstraníme
    window.gMapsLoad = this;
    loadJS("http://maps.googleapis.com/maps/api/js?sensor=false&callback=gMapsLoad._loadStreetView");
    return;
  }
  else
  {
    // v callbacku zrušíme pomocnou globální proměnnou
    window.gMapsLoad = null;
  }
    
  // vytvoříme objekt panorama
  // -- webgl prozatím vypnuto kvůli pomalosti
  this.gPanorama = new google.maps.StreetViewPanorama(this.streetViewDiv,
    {visible: false, position: null, enableCloseButton: false, mode: this._getSvMode()});

  // nastavení polohy a zviditelnění
  this.updatePosition();
  this.gPanorama.setVisible(true);
     
  // add fullscreen button
  this._createCloseButton();     /* Street View v3 has its own close button, but use ours instead */
  this._createFullscreenButton();
          
  // handle clicking on close button
  google.maps.event.addListener(this.gPanorama, 'closeclick', function()
  {
    this._onClickingCloseButton();
  }.bind(this));
           
  // change position - move marker to correct position and set new map center
  google.maps.event.addListener(this.gPanorama, 'position_changed', function(e)
  {
    this.coords = new SMap.Coords.fromGoogle(this.gPanorama.getPosition());
    this.updateArrowPosition();
  }.bind(this));  
          
 // change orientation - rotate arrow
  google.maps.event.addListener(this.gPanorama, 'pov_changed', function(e)
  {
    this.heading = this.gPanorama.getPov().heading;
    this.updateArrowPosition();
  }.bind(this)); 
          
  // call superclass
  this.$super();
           
  // set class for CSS
  add_class(document.documentElement, "svVersion3");
          
  // necessary to sync Google (when after restart), but immediately doing so doesn't work
  window.setTimeout(function()
  {
    this._syncGoogle(); 
  }.bind(this), 0);
          
  // načte všechny kešky
  this.addCaches();   
}

/**
* Přesune se na nové souřadnice
* @param {SMap.Coords} coords Souřadnice
* @param {Int} heading Heading
*/

StreetView.v3.prototype.updatePosition = function(coords, heading)
{
  this.$super(coords, heading);

  this.gPanorama.setPosition(this.coords.toGoogle());
  this.gPanorama.setPov({heading: this.heading, pitch: 0, zoom: 0});
}

/**
* K nastavení šířky okna StreetView přidáme synchronizaci Google mapy
*/

StreetView.v3.prototype._setWidth = function(e, elm)
{
  this.$super(e, elm);

  this._syncGoogle();
}

/**
* Synchronizace Google mapy
*/

StreetView.v3.prototype._syncGoogle = function(e, elm)
{
  google.maps.event.trigger(this.streetViewDiv, "resize");
}

/**
* Určení, který mód pro StreetView použijeme (webgl, html5, html4)
* @returns {String} Mód
*/

StreetView.v3.prototype._getSvMode = function()
{
  // zjištění podpory WebGL
  // zdroj: http://stackoverflow.com/questions/11871077/proper-way-to-detect-webgl-support
  function detectWebGL()
  {
    try
    {
      var canvas = document.createElement( 'canvas' ); 
      return !!window.WebGLRenderingContext && (canvas.getContext('webgl') || canvas.getContext('experimental-webgl'));
    }
    catch(e)
    {
      return false;
    }  
  }
  
  // defaultní verze (záleží na prohlížeči, typicky html4, v Chromu html5)
  // -- pozor, ve Firefoxu je použita defaultně verze html4, která pak vypadá jako "rybí oko"
  var version = "";

  // prohlížeče podporující WebGL
  if (detectWebGL())
    version = "webgl";
    
  // IE11+ (webgl), IE9-10 (html5), IE8 (html4)
  else if (JAK.Browser.client == "ie" && JAK.Browser.version >= 9)
    version = "html5";

  // prozatím nepoužijeme webgl verzi (v IE startuje s prodlevou a je obecně pomalá)
  if (version == "webgl")
    version = "html5";

  return version;
}

/**
* Smažeme všechny markery, co se nalézají ve Street View
*/

StreetView.v3.prototype.deleteAllMarkers = function()
{
  while(this.gMarkers[0])
    this.gMarkers.pop().setMap(null);
}

/**
* Přidá všechny kešky jako markery, nesystémová funkce :(
*/

StreetView.v3.prototype.addCaches = function()
{
  // ukončíme, pokud Street View není aktivní
  if (!this.isActive())
    return;

  this.deleteAllMarkers();
  
  if (!window._caches_)
    return;

  // omezíme počet výsledků
  var added = 0;
  
  var that = this;

  _caches_.forEach(function()
  {
    var image = null;
    var shadow = null;
    
    if (++added > 2500)
      return;
    
    // waymarks
    if (this.isWaymark() && this.wmCategoryIcon)
    {
      image = new google.maps.MarkerImage
      (
        this.wmCategoryIcon,
        new google.maps.Size(20, 34),
        new google.maps.Point(0, 0),    //origin
        new google.maps.Point(-4, -4)   //anchor point
      );
      
      // set size of waymark icons
      image.scaledSize = new google.maps.Size(12, 12);
      
      // set background image (generic marker icon)
      shadow = new google.maps.MarkerImage
      (
        basePath + "Images/Markers/WaymarkGeneric.png",
        new google.maps.Size(20, 34),
        new google.maps.Point(0, 0),  //origin
        new google.maps.Point(0, 0)   //anchor point
      );
    }
    
    // everything else
    else
    {
      image = new google.maps.MarkerImage(basePath + "Images/Markers/" + Cache.css[this.cacheType] + ".png");    
    }
    
    // create marker  
    var marker = new google.maps.Marker
    ({
      position: this.coords.toGoogle(),
      map: that.gPanorama,
      icon: image,
      title: this.name,
      shadow: shadow
    });
    
    that.gMarkers.push(marker);
  });  
}

//***************************************************************************************
//                                Seznam Panorama
//***************************************************************************************

/**
* @class StreetView - Javacriptová verze (v3)
* @group StreetView
*/

StreetView.SeznamPano = JAK.ClassMaker.makeClass({
NAME:"StreetView.SeznamPano",
VERSION:"1.0",
EXTEND: StreetView,
});

/**
* Konstruktor
* @param {SMap} map Mapa
* @param {String} width Šířka okna (25%, 50%, 75%, 100%)
* @param {SMap.Coords} coords Souřadnice startu StreetView
* @param {Int} heading Heading   
*/

StreetView.SeznamPano.prototype.$constructor = function(map, width, coords, heading)
{
  this.panorama = null;

  this.$super.apply(this, arguments);
}

/**
* odstraní z divu Street View
*/

StreetView.SeznamPano.prototype._unloadStreetView = function()
{
   this.$super();

  remove_class(document.documentElement, "svSeznamPano");

  // zrušíme hooky (vypnuto, protože by byl problém, kdyby bylo více instancí této třídy aktivních
  //this._removeHooks();
  
  // destruujeme objekt
  if (this.panorama)
    this.panorama.$destructor();

  // odstraníme všechny zbylé elementy
  while(this.streetViewDiv.firstNode)
    this.streetViewDiv.removeChild(this.streetViewDiv.firstNode);
}
 
/**
* Načte do připraveného divu Street View
* @param {SMap.Coords} coords Souřadnice startu StreetView
*/

StreetView.SeznamPano.prototype._loadStreetView = function()
{
  // Seznam Panorama má vlastní JS a CSS, které je třeba načíst
  // -- učiníme tak pouze jednou za celou session, pak už ne
  if ((!window.SMap || !window.SMap.Pano) && !this.loadedPano)
  {
    this.loadedPano = true;     /* vytvoříme proměnnou, abychom se nemohli zacyklit */

    require(
    [
      "http://api.mapy.cz/js/api/pano.js",      // seznamPath + "js/api/pano.js"
      "http://api.mapy.cz/css/api/pano.css",    // seznamPath + "css/api/pano.css"
      this._loadStreetView.bind(this)    
    ]);

    return;
  }
  
  // navěsíme se na některé API funkce, abychom mohli získávat data o aktuální poloze a headingu
  this._setHooks();

  // Flash doesn't work for file: protocol
  if (window.location.protocol == "file:")
    SMap.Pano.Flash.isSupported = function() {return null;};

  // WebGL doesn't work in Opera 12    
  if (JAK.Browser.client == "opera")
    SMap.Pano.WebGL.isSupported = function() {return null;};    

  // vytvoříme panorama
  var options = {}; /* viz pano-scene.js */
  this.panorama = new SMap.Pano.Scene(this.streetViewDiv, options);
  
  // zjistíme zda je prohlížeč podporován (pokud ne, tak nesmíme volat některé funkce)
  this.isPanoramaSupported = !!SMap.Pano.Renderer.get(this.panorama);  /* je panorama podporováno v tomto prohlížečí? */

  // přidáme tlačítka pro zavření a fullscreen
  this._createCloseButton();
  this._createFullscreenButton();
    
  // call superclass
  this.$super();
  
  // set class for CSS
  add_class(document.documentElement, "svSeznamPano");

  // set position
  this.updatePosition();
}
 
/**
* Přesune se na nové souřadnice
* @param {SMap.Coords} coords Souřadnice
* @param {Int} heading Heading
*/

StreetView.SeznamPano.prototype.updatePosition = function(coords, heading)
{
  this.$super(coords, heading);

  if (!this.isPanoramaSupported)
    return;

  // musíme určit rádius kolem kterého budeme od souřadnic hledat snímek
  // -- čím menší měřítko, tím větší vzdálenost povolíme
  var maxDist = this.map.getScale() / 200;
  
  // pokud jsme nalezli snímek, tak tam panorama přesuneme
  var foundPanorama = function(place)
  {
    this.panorama.show(place, {yaw: this.heading * Math.PI/180});
    
    // nastavíme úhel (funkce výše neupdatuje směr, pokud se změnily souřadnice)
    // -- také je třeba nastavit úhel i ve funkci výše, jinak bude výchozí směr po startu špatný
    this.panorama.setCamera({yaw: this.heading * Math.PI/180 /* v radiánech */});
    
    // aktualizujeme naši polohu a polohu šipky
    this._getCurrentPosition(this.panorama, {coords: true, heading: true});
  }.bind(this);
  
  // pokus jsme nenalezli snímek, tak vrátíme naši pozici zpět
  var notFoundPanorama = function()
  {
    this._getCurrentPosition(this.panorama, {coords: true, heading: true});   
  }.bind(this);  

  SMap.Pano.getBest(this.coords, maxDist).then(foundPanorama, notFoundPanorama);  
}

/**
* K nastavení šířky okna StreetView přidáme synchronizaci panorama
*/

StreetView.SeznamPano.prototype._setWidth = function(e, elm)
{
  this.$super(e, elm);

  if (!this.isPanoramaSupported)
    return;

  this.panorama.syncPort();
}

/**
* Je třeba se navěsit na některé funkce API, abychom mohli získávat údaje o poloze a headingu jak se mění
*/

StreetView.SeznamPano.prototype._setHooks = function(panorama)
{
  var that = this;

  // navěsíme se na funkci show, která je volána při změně polohy  
  this.oldShow = SMap.Pano.Scene.prototype.show;
  
  SMap.Pano.Scene.prototype.show = function()
  {
    var ret = that.oldShow.apply(this, arguments);
    
    // zde nesmíme updatovat heading, protože jinak nefunguje klik na šipku za účelem změny směru 
    that.map.getSignals().makeEvent("seznam-panorama-change", that, {panorama: this, type: {coords: true, heading: false}}); 
    return ret;
  }
  
  // ================================================
  // navěsíme se na funkci setCamera, která je volána při změně headingu
  this.oldSetCamera = SMap.Pano.Scene.prototype.setCamera;
  
  SMap.Pano.Scene.prototype.setCamera = function()
  {
    var ret = that.oldSetCamera.apply(this, arguments);
    that.map.getSignals().makeEvent("seznam-panorama-change", that, {panorama: this, type: {coords: false, heading: true}}); 
    return ret;
  }

  // ================================================
  // navěstíme se na událost vytvořenou v předchozích funkcích
  this._sc.push(this.map.getSignals().addListener(window, "seznam-panorama-change", function(data)
  {
    this._getCurrentPosition(arguments[0].data.panorama, arguments[0].data.type);
  }.bind(this)));
  
  // ================================================  
  // opravíme načítání circle.png a circle-active.png, aby je načítalo ze serveru, jinak hodí CORS chybu
  this.old_createCircles = SMap.Pano.WebGL.prototype._createCircles;
  
  SMap.Pano.WebGL.prototype._createCircles = function()
  {
    var path = SMap.CONFIG.img;
    SMap.CONFIG.img = "http://api.mapy.cz/img/api";
    var ret = that.old_createCircles.apply(this, arguments);
    SMap.CONFIG.img = path;
    return ret;
  }
}

/**
* Odstranění hooků
*/

StreetView.SeznamPano.prototype._removeHooks = function()
{
  SMap.Pano.Scene.prototype.show = this.oldShow;
  SMap.Pano.Scene.prototype.setCamera = this.oldSetCamera;
  SMap.Pano.WebGL.prototype._createCircles = this.old_createCircles;
}

/**
* Handler pro změnu polohy či headingu panorama; updatujeme souřadnice nebo heading, a šipku
* @param {SMap.Pano.Scene} panorama Objekt panorama
* @param {Objekt} whatToUpdate Co všechno načíst z panorama (souřadnice, heading, nebo obojí)
*/

StreetView.SeznamPano.prototype._getCurrentPosition = function(panorama, whatToUpdate)
{
  // tato funkce má parametr proto, abychom vyloučili možnost, že byl hook
  // přepsán jinou instancí této třídy a dostali jsme tak panorama nám nepříslušející  
  if (panorama !== this.panorama)
    return;

  if (whatToUpdate.coords)
    this.coords = panorama.getPlace().getCoords();
  
  if (whatToUpdate.heading)
    this.heading = panorama.getCamera().yaw * 180/Math.PI;

  this.updateArrowPosition();  
}


//****************************************************************************************************
//************************          handler pro kontextové menu               ************************
//****************************************************************************************************
//------------------------------------------------
function runStreetView(coords, type)
{
  // převod ID kontextového menu na název třídy
  var types = 
  {
    cm_street_view_v3: "v3", 
    cm_street_view_v2: "Flash",
    cm_street_view_embed: "Embedded",
    cm_seznam_pano: "SeznamPano"
  };

  // pokud pouštíme funkci poprvé, pak nainstalujeme handler
  if (!window.svObjekt)
  {
    // při změně velikosti Street View pomocí tlačítka novou velikost uložíme
    m.getSignals().addListener(window, "street-view-resize", function(data)
    {
      settings.streetview_width = arguments[0].data.newWidth;
    });
  }
  
  // pokud je StreetView aktivní, ale jiná než požadovaná verze, tak ji nejdříve vypneme
  if (window.svObjekt && window.svObjekt.isActive() 
  && !(window.svObjekt instanceof StreetView[types[type]]))
  {
    window.svObjekt.$destructor();
  }  

  // pokud pouštíme funkci poprvé nebo je StreetView vypnuté, vytvoříme nový objekt
  if (!window.svObjekt                 /* pouštíme funkci poprvé */
    || !window.svObjekt.isActive())    /* StreetView je vypnuté */
  {
    window.svObjekt = new StreetView[types[type]](m, settings.streetview_width, coords, 0);
  }
  
  // pokud je StreetView aktivní a správná verze, pak ho jen posuneme na nové souřadnice
  else
  {
    window.svObjekt.updatePosition(coords);
  }  
}